"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Fetch = void 0;
var query_1 = require("./query");
var conditions = require("./conditions/index");
var Fetch = /** @class */ (function (_super) {
    __extends(Fetch, _super);
    function Fetch(type, key) {
        var _this = _super.call(this) || this;
        _this.conditionChain = [];
        _this.setActionName('fetch');
        _this.setType(type);
        _this.setKey(key);
        return _this;
    }
    Fetch.prototype.select = function (columns) {
        this.conditionChain.push(new conditions.SelectCondition(columns));
        return this;
    };
    Fetch.prototype.orWhere = function (condition) {
        this.conditionChain.push(new conditions.OrWhereCondition(condition));
        return this;
    };
    Fetch.prototype.orWhereDoesntHave = function (relation, condition) {
        var q = new Fetch(relation, Fetch.makeUUID());
        this.conditionChain.push(new conditions.OrWhereDoesntHaveCondition(relation, condition(q)));
        return this;
    };
    Fetch.prototype.scope = function (name, params) {
        this.conditionChain.push(new conditions.ScopeCondition(name, params));
        return this;
    };
    Fetch.prototype.where = function (condition) {
        this.conditionChain.push(new conditions.WhereCondition(condition));
        return this;
    };
    Fetch.prototype.whereHas = function (relation, condition) {
        var q = new Fetch(relation, Fetch.makeUUID());
        var subCondition = typeof condition === 'function' ? condition(q) : null;
        this.conditionChain.push(new conditions.WhereHasCondition(relation, subCondition));
        return this;
    };
    Fetch.prototype.orWhereHas = function (relation, condition) {
        var q = new Fetch(relation, Fetch.makeUUID());
        this.conditionChain.push(new conditions.OrWhereHasCondition(relation, condition(q)));
        return this;
    };
    Fetch.prototype.whereDoesntHave = function (relation, condition) {
        var q = new Fetch(relation, Fetch.makeUUID());
        this.conditionChain.push(new conditions.WhereDoesntHaveCondition(relation, condition(q)));
        return this;
    };
    Fetch.prototype.whereIn = function (key, values) {
        this.conditionChain.push(new conditions.WhereInCondition(key, values));
        return this;
    };
    Fetch.prototype.whereNotIn = function (key, values) {
        this.conditionChain.push(new conditions.WhereNotInCondition(key, values));
        return this;
    };
    Fetch.prototype.whereNull = function (columnName) {
        this.conditionChain.push(new conditions.WhereNullCondition(columnName));
        return this;
    };
    Fetch.prototype.whereNotNull = function (columnName) {
        this.conditionChain.push(new conditions.WhereNotNullCondition(columnName));
        return this;
    };
    Fetch.prototype.orWhereNull = function (columnName) {
        this.conditionChain.push(new conditions.OrWhereNullCondition(columnName));
        return this;
    };
    Fetch.prototype.orWhereNotNull = function (columnName) {
        this.conditionChain.push(new conditions.OrWhereNotNullCondition(columnName));
        return this;
    };
    Fetch.prototype.whereColumn = function (columnsList) {
        this.conditionChain.push(new conditions.WhereColumnCondition(columnsList));
        return this;
    };
    Fetch.prototype.whereExists = function (select, from, where) {
        this.conditionChain.push(new conditions.WhereExistsCondition(select, from, where));
        return this;
    };
    Fetch.prototype.with = function (tableNameList) {
        this.conditionChain.push(new conditions.WithCondition(tableNameList));
        return this;
    };
    Fetch.prototype.limit = function (limit) {
        this.conditionChain.push(new conditions.LimitCondition(limit));
        return this;
    };
    Fetch.prototype.offset = function (offset) {
        this.conditionChain.push(new conditions.OffsetCondition(offset));
        return this;
    };
    Fetch.prototype.paginate = function (pageNumber, perPage) {
        this.pageNumber = pageNumber;
        this.perPage = perPage;
        return this;
    };
    Fetch.prototype.render = function () {
        var result = [];
        this.conditionChain.forEach(function (cond) {
            result.push(cond.render());
        });
        var pagination = {};
        if (this.perPage) {
            pagination = { per_page: this.perPage };
        }
        if (this.pageNumber) {
            pagination = __assign(__assign({}, pagination), { page: this.pageNumber });
        }
        return {
            query: this.getActionName(),
            key: this.getKey(),
            type: this.getType(),
            params: __assign(__assign({}, pagination), { parts: result }),
        };
    };
    return Fetch;
}(query_1.Query));
exports.Fetch = Fetch;
